#! /bin/bash

LOCAL_DIR="A"
REMOTE_DIR="B"

rsync -rn --ignore-existing --out-format='%n' "$LOCAL_DIR"/ "$REMOTE_DIR"/ \
| grep -v "skipping non-regular file" | grep -E -v '/$' | sed -e 's/^/L   /'

rsync -rn --ignore-existing --out-format='%n' "$REMOTE_DIR"/ "$LOCAL_DIR"/ \
| grep -v "skipping non-regular file" | grep -E -v '/$' | sed -e 's/^/R   /'

readarray -t common_file_list < <(rsync -rn --existing --out-format='%n' \
  "$LOCAL_DIR"/ "$REMOTE_DIR"/ | grep -v "skipping non-regular file")

readarray -t common_files_with_diff_sizes_list < <(rsync -rn --existing --out-format='%n' \
  --size-only "$LOCAL_DIR"/ "$REMOTE_DIR"/ | grep -v "skipping non-regular file")

readarray -t common_files_with_diff_sizes_list_newest_on_local \
  < <(rsync -rn --update --out-format='%n' \
  --files-from=<( printf "%s\n" "${common_files_with_diff_sizes_list[@]}" ) \
  "$LOCAL_DIR"/ "$REMOTE_DIR"/ | grep -v "skipping non-regular file")

readarray -t common_files_with_diff_sizes_list_newest_on_remote \
  < <(rsync -rn --update --out-format='%n' \
  --files-from=<( printf "%s\n" "${common_files_with_diff_sizes_list[@]}" ) \
  "$REMOTE_DIR"/ "$LOCAL_DIR"/ | grep -v "skipping non-regular file")

if [[ -n ${common_files_with_diff_sizes_list_newest_on_local[@]} ]]; then
  printf "Sl  %s\n" "${common_files_with_diff_sizes_list_newest_on_local[@]}"
fi
if [[ -n ${common_files_with_diff_sizes_list_newest_on_remote[@]} ]]; then
  printf "Sr  %s\n" "${common_files_with_diff_sizes_list_newest_on_remote[@]}"
fi

common_files_with_equal_size_list=()
for f in "${common_file_list[@]}"
do
  if printf '%s\0' "${common_files_with_diff_sizes_list[@]}" \
    | grep -Fxqz -- "$f"; then
    # If the current file has different sizes on both sides, skip it.
    continue;
  fi

  common_files_with_equal_size_list+=( "$f" )
done

readarray -t common_files_with_same_size_and_diff_content \
  < <(rsync -n --checksum --out-format='%n' \
  --files-from=<( printf "%s\n" "${common_files_with_equal_size_list[@]}" ) \
  "$LOCAL_DIR"/ "$REMOTE_DIR"/)

rsync -rn --update --out-format='%n' \
  --files-from=<( printf "%s\n" "${common_files_with_same_size_and_diff_content[@]}" ) \
  "$LOCAL_DIR"/ "$REMOTE_DIR"/ | sed -e 's/^/Cl  /'

rsync -rn --update --out-format='%n' \
  --files-from=<( printf "%s\n" "${common_files_with_same_size_and_diff_content[@]}" ) \
  "$REMOTE_DIR"/ "$LOCAL_DIR"/ | sed -e 's/^/Cr  /'
